/*
 * Copyright (c) 1998 The Santa Cruz Operation, Inc.. All Rights Reserved. 
 *                                                                         
 *        THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF THE               
 *                   SANTA CRUZ OPERATION INC.                             
 *                                                                         
 *   The copyright notice above does not evidence any actual or intended   
 *   publication of such source code.                                      
 */

#ifndef _UNISTD_H
#define _UNISTD_H
#ident	"@(#)sgs-head:common/head/unistd.h	1.52"

#include <sys/types.h>
#include <sys/unistd.h>

#ifndef _USECONDS_T
#define _USECONDS_T
typedef unsigned long useconds_t;
#endif

#if !defined(_XOPEN_SOURCE) \
	&& !defined(_POSIX_SOURCE) && !defined(_POSIX_C_SOURCE)

#define	GF_PATH	"/etc/group"	/* Path name of the "group" file */
#define	PF_PATH	"/etc/passwd"	/* Path name of the "passwd" file */

#endif

#if defined (_XOPEN_SOURCE) && _XOPEN_SOURCE_EXTENDED - 0 >= 1 \
	|| !defined(_XOPEN_SOURCE) \
		&& !defined(_POSIX_SOURCE) && !defined(_POSIX_C_SOURCE)

#define F_ULOCK	0	/* Unlock a previously locked region */
#define F_LOCK	1	/* Lock a region for exclusive use */
#define F_TLOCK	2	/* Test and lock a region for exclusive use */
#define F_TEST	3	/* Test a region for other processes locks */

#endif

#define _POSIX_JOB_CONTROL	1
#define _POSIX_SAVED_IDS	1

#ifndef _POSIX_VDISABLE
#define _POSIX_VDISABLE		0
#endif

#ifndef	NULL
#define NULL	0
#endif

#define	STDIN_FILENO	0
#define STDOUT_FILENO	1
#define STDERR_FILENO	2

#define _XOPEN_UNIX	1
#define _XOPEN_ENH_I18N		1
#define _XOPEN_XPG4		1
#define _POSIX2_C_VERSION	199209L
#define _POSIX2_VERSION		199209L
#define _XOPEN_XCU_VERSION	4

#define _POSIX2_C_BIND		1
#define _POSIX2_CHAR_TERM	1
#define _POSIX2_FORT_RUN	1
#define _POSIX2_LOCALEDEF	1
#define _POSIX2_UPE		1

#define _LFS_ASYNCHRONOUS_IO	1
#define _LFS_LARGEFILE		1
#define _LFS64_ASYNCHRONOUS_IO	1
#define _LFS64_LARGEFILE	1
#define _LFS64_STDIO		1

#ifdef __cplusplus
extern "C" {
#endif

extern int	access(const char *, int);
extern unsigned	alarm(unsigned);
extern int	chdir(const char *);
extern int	chown(const char *, uid_t, gid_t);
extern int	close(int);
extern char	*cuserid(char *);
extern int	dup(int);
extern int	dup2(int, int);
extern int	execl(const char *, const char *, ...);
extern int	execle(const char *, const char *, ...);
extern int	execlp(const char *, const char *, ...);
extern int	execv(const char *, char *const *);
extern int	execve(const char *, char *const *, char *const *);
extern int	execvp(const char *, char *const *);
extern void	_exit(int);
extern pid_t	fork(void);
extern long	fpathconf(int, int);
extern char	*getcwd(char *, size_t);
extern gid_t	getegid(void);
extern uid_t	geteuid(void);
extern gid_t	getgid(void);
extern int	getgroups(int, gid_t *);
extern char	*getlogin(void);
extern pid_t	getpgrp(void);
extern pid_t	getpid(void);
extern pid_t	getppid(void);
extern uid_t	getuid(void);
extern int	isatty(int);
extern int	link(const char *, const char *);
extern n_off_t	lseek(int, n_off_t, int);
extern long	pathconf(const char *, int);
extern int	pause(void);
extern int	pipe(int *);
extern ssize_t	read(int, void *, size_t);
extern int	rmdir(const char *);
extern int	setgid(gid_t);
extern int	setpgid(pid_t, pid_t);
extern pid_t	setsid(void);
extern int	setuid(uid_t);
extern unsigned	sleep(unsigned);
extern long	sysconf(int);
extern pid_t	tcgetpgrp(int);
extern int	tcsetpgrp(int, pid_t);
extern char	*ttyname(int);
extern int	unlink(const char *);
extern ssize_t	write(int, const void *, size_t);

#if defined(_XOPEN_SOURCE) || (_POSIX_C_SOURCE - 0 > 1) \
	|| !defined(_POSIX_SOURCE) && !defined(_POSIX_C_SOURCE)

extern size_t	confstr(int, char *, size_t);
extern int	getopt(int, char *const*, const char *);
extern char	*optarg;
extern int	optind, opterr, optopt;

#endif

#if defined(_XOPEN_SOURCE) \
	|| !defined(_POSIX_SOURCE) && !defined(_POSIX_C_SOURCE)

extern int	chroot(const char *);
extern char	*crypt(const char *, const char *);
extern char	*ctermid(char *);	/* REALLY OKAY HERE? */
extern void	encrypt(char *, int);
extern int	fsync(int);
extern char	*getpass(const char *);
extern int	nice(int);
extern void	swab(const void *, void *, ssize_t);

#endif

#if defined (_XOPEN_SOURCE) && _XOPEN_SOURCE_EXTENDED - 0 >= 1 \
	|| !defined(_XOPEN_SOURCE) \
		&& !defined(_POSIX_SOURCE) && !defined(_POSIX_C_SOURCE)

extern int	brk(void *);
extern int	fchown(int, uid_t, gid_t);
extern int	fchdir(int);
extern int	ftruncate(int, n_off_t);
extern int	getdtablesize(void);
extern long	gethostid(void);
extern int	gethostname(char *, size_t);
extern int	getpagesize(void);
extern pid_t	getpgid(pid_t);
extern pid_t	getsid(pid_t);
extern char	*getwd(char *);
extern int	lchown(const char *, uid_t, gid_t);
extern int	lockf(int, int, long);
extern int	readlink(const char *, char *, size_t);
extern void	*sbrk(int);
extern pid_t	setpgrp(void);
extern int	setregid(gid_t, gid_t);
extern int 	setreuid(uid_t, uid_t);
extern int	symlink(const char *, const char *);
extern void	sync(void);
extern int	truncate(const char *, n_off_t);
extern useconds_t ualarm(useconds_t, useconds_t);
extern int 	usleep(useconds_t);
extern pid_t	vfork(void);

#endif

#if defined(_LARGEFILE64_SOURCE) || defined(_FILE_OFFSET_BITS) \
	|| !defined(_XOPEN_SOURCE) && !defined(_POSIX_SOURCE) \
		&& !defined(_POSIX_C_SOURCE)

extern int	ftruncate32(int, off32_t);
extern off32_t	lseek32(int, off32_t, int);
extern ssize_t	pread32(int, void *, size_t, off32_t);
extern ssize_t	pwrite32(int, const void *, size_t, off32_t);
extern int	truncate32(const char *, off32_t);

extern int	ftruncate64(int, off64_t);
extern off64_t	lseek64(int, off64_t, int);
extern ssize_t	pread64(int, void *, size_t, off64_t);
extern ssize_t	pwrite64(int, const void *, size_t, off64_t);
extern int	truncate64(const char *, off64_t);

#endif

#if !defined(_XOPEN_SOURCE) \
	&& !defined(_POSIX_SOURCE) && !defined(_POSIX_C_SOURCE)

extern int	acct(const char *);
extern void	exit(int);
extern int	fattach(int, const char *);
extern int	fdetach(const char *);
extern pid_t	fork1(void);
extern pid_t	forkall(void);
extern char	*getpass_r(const char *, char *, size_t);
extern char	*gettxt(const char *, const char *);
extern int	ioctl(int, int, ...);
extern int	mincore(caddr_t, size_t, char *);
extern ssize_t	pread(int, void *, size_t, n_off_t);
extern void	profil(unsigned short *, unsigned, unsigned, unsigned);
extern int	ptrace(int, pid_t, int, int);
extern ssize_t	pwrite(int, const void *, size_t, n_off_t);
extern int	rename(const char *, const char *);
extern int	setgroups(int, const gid_t *);
extern int	stime(const time_t *);
extern int	_xgetlogin_r(char *, size_t);
extern int	_xttyname_r(int, char *, size_t);

#ifdef _SIMPLE_R

extern char	*getlogin_r(char *, size_t);
extern char	*ttyname_r(int, char *, size_t);

#elif defined(_EFTSAFE)

#undef getlogin_r
#define getlogin_r(p,s)		_xgetlogin_r(p,s)
#undef ttyame_r
#define ttyname_r(f,p,s)	_xttyname_r(f,p,s)

#else

#ifndef _GETLOGIN_R
#define _GETLOGIN_R
static int
getlogin_r(char *__1, size_t __2)
{
	return _xgetlogin_r(__1, __2);
}
#endif /*_GETLOGIN_R*/

#ifndef _TTYNAME_R
#define _TTYNAME_R
static int
ttyname_r(int __1, char *__2, size_t __3)
{
	return _xttyname_r(__1, __2, __3);
}
#endif /*_TTYNAME_R*/

#endif /*_SIMPLE_R*/

#endif /*!defined(_XOPEN_SOURCE) && ...*/

#if _FILE_OFFSET_BITS - 0 == 32
#undef ftruncate
#define ftruncate	ftruncate32
#undef lseek
#define lseek		lseek32
#undef pread
#define pread		pread32
#undef pwrite
#define pwrite		pwrite32
#undef truncate
#define truncate	truncate32
#elif _FILE_OFFSET_BITS - 0 == 64
#undef ftruncate
#define ftruncate	ftruncate64
#undef lseek
#define lseek		lseek64
#undef pread
#define pread		pread64
#undef pwrite
#define pwrite		pwrite64
#undef truncate
#define truncate	truncate64
#elif defined(_FILE_OFFSET_BITS)
#error "_FILE_OFFSET_BITS, if defined, must be 32 or 64"
#endif /*_FILE_OFFSET_BITS*/

#ifdef __cplusplus
}
#endif

#endif /*_UNISTD_H*/
